package lufyt.plot;

/**
 * Abstract base-class for representation of one-dimensional
 * plot-ranges along with a scale-transform.
 */
public abstract class Range {
    
    /** Lower bount of the range. */
    public double min;
    /** Upper bount of the range. */
    public double max;
    /** Transforms data-coordinates to intermediate axis-coordinates */
    public abstract double transform(double x);
    /** Inverse function of transform. */
    public abstract double inverse(double f);
    /** Returns a reader object for read-only access to this Range. */
    public Reader reader() {return new Reader(this);}
    /** Returns a Scale object suitable for this Range */
    public abstract Scale getDefaultScale();
    
    /**
     * Class to provide read-only access for a current Range
     * object. This class is final to prevent acess to data-altering
     * methods of the current Range. */
    public static final class Reader {
	private Range range;
	/** Constructor that sets r to the current Range of this reader. */
	public Reader(Range r) {set(r);}
	/** Sets the current Range of this reader. */
	public void set(Range r) {range = r;}
	public double getMin() {return range.min;}
	public double getMax() {return range.max;}
	public double transform(double x) {return range.transform(x);}
	public double inverse(double f) {return range.inverse(f);}
    }

    /** Concetization of Range for linear plotting with a unity tranform. */
    public static class Lin extends Range {
	public double transform(double x) {return x;}
	public double inverse(double f) {return f;}
	public Scale getDefaultScale() {return new Scale.Lin();}
    }
    /** Concetization of Range for logarithmic plotting. */
    public static class Log extends Range {
	public double transform(double x) {return Math.log(x);}
	public double inverse(double f) {return Math.exp(f);}
	public Scale getDefaultScale() {return new Scale.Log();}
    }
}
